//---------------------------------------------------------------------------------
//							    SPI Driver
//				   Write By Mohammad mazarei @ 1390/7/2
//					Email: mohammad.mazarei@gmail.com
//					 Publish In IranMicro.ir
//---------------------------------------------------------------------------------

//------------------------------------------------------------------------------
///
///  !
///
/// -# Enable the SPI pins required by the application (see pio.h).
/// -# Configure the SPI using the SPI_Configure function. This enables the
///    peripheral clock. The mode register is loaded with the given value.
/// -# Configure all the necessary chip selects with SPI_ConfigureNPCS.
/// -# Enable the SPI by calling SPI_Enable.
/// -# Send/receive data using SPI_Write and SPI_Read. Note that SPI_Read
///    must be called after SPI_Write to retrieve the last value read.
/// -# Send/receive data using the PDC with the SPI_WriteBuffer and
///    SPI_ReadBuffer functions.
/// -# Disable the SPI by calling SPI_Disable.
//------------------------------------------------------------------------------

#ifndef SPI_H
#define SPI_H


//------------------------------------------------------------------------------
//         Definitions
//------------------------------------------------------------------------------
//------------------------------ SPI PIO -----------------------
#define SPI0_PORT	AT91C_BASE_PIOA
#define SPI0_NPCS0		12
#define SPI0_NPCS1		13
#define SPI0_NPCS2		14
#define SPI0_NPCS3		15
#define SPI0_MISO		16
#define SPI0_MOSI		17
#define SPI0_SPCK		18
#define SPI0_PIO_MASK	((1<<SPI0_NPCS0)|(1<<SPI0_NPCS1)|(1<<SPI0_NPCS2)|(1<<SPI0_NPCS3)|(1<<SPI0_MISO)|(1<<SPI0_MOSI)|(1<<SPI0_SPCK))

//------------------Global SETTING  MR REG---------------------
#define  SPI_SLAVE			(0<<0) //  Set SLAVE MODE
#define  SPI_MASTER 		(1<<0) //  Set MASTER SPI
#define  SPI_MULTI_DEVICE	(1<<1) //  SET MULTI DEVICE SUPORT
#define  SPI_ONE_DEVIDE  	(0<<1) //  SET ONE Device Suort
#define  SPI_EN_DECODER		(1<<2) //  Chip Select Decode
#define  SPI_MOD_FDIS       (1<<4) //  Disable Mode Fault Detection
#define  SPI_LLB			(1<<7) //  ENABLE INTRNAL FEEDBACK

//---------------------- CHANAL SETTING ------------------------
 // SPI MODE
#define SPI_MODE_0			2 // L ACRIVE CLOCK & POS EDGE
#define SPI_MODE_1			0 // L ACTIVE ClOCK & NEG EDGE
#define SPI_MODE_2			3 // H ACTIVE CLOCK & NEG EDGE
#define SPI_MODE_3			1 // H ACTIVE CLOCK & POS EDGE

 // DATA LEN IN SPI
#define SPI_8_BIT			(0<<4)
#define SPI_9_BIT			(1<<4)
#define SPI_10_BIT			(2<<4)
#define SPI_11_BIT			(3<<4)
#define SPI_12_BIT			(4<<4)
#define SPI_13_BIT			(5<<4)
#define SPI_14_BIT			(6<<4)
#define SPI_15_BIT			(7<<4)
#define SPI_16_BIT			(8<<4)
//------------------------------------------------------------------------------
/// "SPI configuration macros"
//------------------------------------------------------------------------------

#define SPI_Configure_PIO_BIT(SPI_PORT,SPI_PIN)		SPI_PORT->PIO_PDR=(1<<SPI_PIN);SPI_PORT->PIO_MDER=(1<<SPI_PIN);SPI_PORT->PIO_ASR=(1<<SPI_PIN);SPI_PORT->PIO_PPUER=(1<<SPI_PIN)
#define SPI_Configure_PIO_MAKS(SPI_PORT,SPI_MASK)	SPI_PORT->PIO_PDR=SPI_MASK;SPI_PORT->PIO_MDER=SPI_MASK;SPI_PORT->PIO_ASR=SPI_MASK;SPI_PORT->PIO_PPUER=SPI_MASK


/// Calculate the PCS field value given the chip select NPCS value
#define SPI_PCS(npcs)       ((~(1 << npcs) & 0xF) << 16)

/// Calculates the value of the CSR SCBR field given the baudrate and MCK.
#define SPI_SCBR(baudrate, masterClock) \
            ((unsigned int) (masterClock / baudrate) << 8)

/// Calculates the value of the CSR DLYBS field given the desired delay (in ns)
#define SPI_DLYBS(delay, masterClock) \
            ((unsigned int) (((masterClock / 1000000) * delay) / 1000) << 16)

/// Calculates the value of the CSR DLYBCT field given the desired delay (in ns)
#define SPI_DLYBCT(delay, masterClock) \
            ((unsigned int) (((masterClock / 1000000) * delay) / 32000) << 16)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//         Exported functions
//------------------------------------------------------------------------------

extern void SPI_Enable(AT91S_SPI *spi);
extern void SPI_Disable(AT91S_SPI *spi);
extern void SPI_Configure(AT91S_SPI *spi,
                                 unsigned int id,
                                 unsigned int configuration);
extern void SPI_ConfigureNPCS(AT91S_SPI *spi,
                                     unsigned int npcs,
                                     unsigned int configuration);

extern void SPI_Write(AT91S_SPI *spi, unsigned int npcs, unsigned short data);
extern unsigned char SPI_WriteBuffer(AT91S_SPI *spi,
                                            void *buffer,
                                            unsigned int length);

extern unsigned char SPI_IsFinished(AT91S_SPI *pSpi);

extern unsigned short SPI_Read(AT91S_SPI *spi);
extern unsigned char SPI_ReadBuffer(AT91S_SPI *spi,
                                           void *buffer,
                                           unsigned int length);


#endif

